@@ -4,26 +4,48 @@ Dictionary of several parameters concerning Detectors, Materials, Transforms etc
 that are used in LaueTools and in LaueToolsGUI module
 
 Lauetools project
-2009 - October 2024
+2009 - October 2025
 
 """
 __author__ = "Jean-Sebastien Micha, CRG-IF BM32 @ ESRF"
 
 import copy
-import os
+import os, json
 import re
+from importlib import resources
 
 import numpy as np
 
+USE_MATERIALS_LIBRARY = True
+USE_USER_MATERIALS_YAML_FILE = True
+
 LAUETOOLSFOLDER = os.path.split(__file__)[0]
+DEFAULT_MATERIALS_FILE = os.path.join(LAUETOOLSFOLDER, "materials.yaml")
 # print("LaueToolsProjectFolder", LAUETOOLSFOLDER)
 # WRITEFOLDER = os.path.join(LAUETOOLSFOLDER, "laueanalysis")
 
+def _load_config():
+    cfg_file = os.path.join(LAUETOOLSFOLDER, "lauetools_config.json")
+    if os.path.exists(cfg_file):
+        with open(cfg_file) as f:
+            return json.load(f)
+    return {}
+
 #####   materials.yaml    #####
 import yaml
 
+def get_materials_file():
+    """Return path to active materials.yaml, considering saved user choice."""
+    cfg = _load_config()
+    if "materials_dir" in cfg:
+        user_path = os.path.join(cfg["materials_dir"], "materials.yaml")
+        if os.path.exists(user_path):
+            print('Using user materials.yaml: ', user_path)
+            return user_path
+
+
 class Materials:
-    def __init__(self, key_material=None, yaml_file_path=None):
+    def __init__(self, key_material=None, yaml_file_path=DEFAULT_MATERIALS_FILE, use_user_yaml_file=USE_USER_MATERIALS_YAML_FILE):
         """
         Initialize the Materials with the YAML file.
 
@@ -32,12 +54,22 @@ class Materials:
         # Example usage:
         ll =  =DictLT.Materials('Fe')
         ==> ['Fe', [2.856, 2.856, 2.856, 90, 90, 90], 'bcc']
-        # library = Materials('materials.yaml')
+        # library = Materials(yaml_file_path='materials.yaml')
         # print(library.get_material_dict())
         # print(library.get_material('Ag'))
         # print(library.get_material_as_list('Ag'))
         """
         self.yaml_file_path = yaml_file_path
+        self.use_user_yaml_file = use_user_yaml_file
+        
+        if use_user_yaml_file:
+            try:
+                yaml_file_path = get_materials_file()
+            except:
+                print('No user copy version of materials.yaml found. Using default materials.yaml')
+                yaml_file_path = DEFAULT_MATERIALS_FILE
+            self.yaml_file_path = yaml_file_path
+
         self.key_material = key_material
         
         self.read_yaml()
@@ -124,11 +156,29 @@ class Materials:
         """
         return self.materials
 
-    def read_yaml(self, yaml_file_path=None):
+    def read_yaml(self, yaml_file_path=None, use_user_yaml_file=USE_USER_MATERIALS_YAML_FILE):
+        """
+        Read a YAML file containing materials data.
+
+        If yaml_file_path is None, read the YAML file specified by self.yaml_file_path.
+        If use_user_yaml_file is True, try to read the user's materials.yaml file copy and editable version whose location is saved in lauetools_config.json file.
+        If no user materials.yaml file is found, read the default materials.yaml file.
+
+        :param yaml_file_path: Path to the YAML file to read.
+        :param use_user_yaml_file: If True, try to read the user's materials.yaml file.
+        """
         if yaml_file_path is None:
             yaml_file_path = self.yaml_file_path
+        if not(use_user_yaml_file):
+            try:
+                yaml_file_path = get_materials_file()
+            except:
+                print('No user materials.yaml found. Using default materials.yaml')
+                yaml_file_path = DEFAULT_MATERIALS_FILE
+
         with open(yaml_file_path, 'r') as file:
             self.materials = yaml.safe_load(file)
+            self.yaml_file_path = yaml_file_path
 
     def check_material(self, key_material):
         if key_material in self.materials.keys():
@@ -189,10 +239,12 @@ class Materials:
             return None
         return material['extinction']
 
-    def add_or_update_material(self, label, lattice, extinction):
+    def add_or_update_material(self, label, lattice, extinction, yaml_file_path=None):
         """
         Add or update a material in the dictionary and update the YAML file.
 
+        resets yaml_file_path if yaml_file_path is not None
+
         :param label: String label of the material.
         :param lattice: List of lattice parameters.
         :param extinction: String extinction rule.
@@ -202,7 +254,8 @@ class Materials:
             'lattice': lattice,
             'extinction': extinction
         }
-
+        if yaml_file_path is not None:
+            self.yaml_file_path = yaml_file_path
         with open(self.yaml_file_path, 'w') as file:
             yaml.dump(self.materials, file, sort_keys=False, default_flow_style=False)
 
@@ -221,25 +274,13 @@ class Materials:
 import os
 from importlib import resources
 
-def get_materials_file():
-    """
-    Returns the path to the active materials.yaml:
-    - User copy (~/.lauetools/materials.yaml) if present
-    - Otherwise, the packaged default version
-    """
-    user_path = os.path.join(os.path.expanduser("~"), ".Lauetools", "materials.yaml")
-    if os.path.exists(user_path):
-        return user_path
-    else:
-        with resources.path(LAUETOOLSFOLDER, "materials.yaml") as default_path:
-            return str(default_path)
-
 yaml_file_path = get_materials_file()
 
 # either use of yaml_file_path
-dict_Materials = Materials(yaml_file_path=yaml_file_path)
+if USE_MATERIALS_LIBRARY:
+    dict_Materials = Materials()
 # or old usual dictionary
-if 1:
+elif not(USE_MATERIALS_LIBRARY):
     #------------------------------------------------------------------------
     # --- -----------  HARD CODED Element-materials library
     #-------------------------------------------------------------------------
